#include <iostream>
#include <mpi.h>
#include <blitz/array.h>

using namespace std;
using namespace blitz;

int main(int argc, char *argv[])
{
	MPI_Init(&argc, &argv);

	int my_id;
	int numprocs;
	int N;
	int local_N;
	int max;

	Array<int, 1> array_input;
	Array<int, 1> array;
	Array<int, 1> array_max;

	MPI_Comm_rank(MPI_COMM_WORLD, &my_id);
	MPI_Comm_size(MPI_COMM_WORLD, &numprocs);

	//Master process reads the data and send it to all other process
	if (my_id == 0) {

		//Continue to take N untill it is divided by numprocs
		cin >> N;

		array_input.resize(N);

		//Input N numbers
		for (int i=0; i<N; i++) {
			cin >> array_input(i);
		}

		/*Output array for testing

		cout << N << endl;
		for (int i=0; i<N; i++) {
			cout << array_input(i) << "\t";
		}
		cout << endl;
		*/
	}


	//Broadcast N to all procs
	MPI_Bcast(&N, 1, MPI_INT, 0, MPI_COMM_WORLD);

	local_N = N/numprocs;
	array.resize(local_N);

	MPI_Scatter(array_input.data(), local_N, MPI_INT, array.data(), local_N, MPI_INT, 0, MPI_COMM_WORLD);

	/*Output array for testing

	for (int p=0; p<numprocs; p++) {
		if (my_id == p) {
			cout << local_N << endl;
			for (int i=0; i<array.size(); i++) {
				cout << array(i) << "\t";
			}
			cout << endl;
		}
		MPI_Barrier(MPI_COMM_WORLD);
	}
	*/

	//Find maximum
	int local_max=0;

	for (int i=0; i<local_N; i++)
		if (array(i) > local_max)
			local_max = array(i);

	// cout << my_id << ", " << local_max << endl;

	//Collect maximum from all process in master process
	array_max.resize(numprocs);

	MPI_Gather(&local_max, 1, MPI_INT, array_max.data(), 1, MPI_INT, 0, MPI_COMM_WORLD);


	if (my_id == 0) {

		/*Output array for testing
		for (int i=0; i<numprocs; i++) {
			cout << array_max(i) << "\t";
		}
		cout << endl;
		*/

		int max=0;

		for (int i=0; i<numprocs; i++)
			if (array_max(i) > max)
				max = array_max(i);


		cout << "Maximum number = " << max << endl;

	}

	MPI_Finalize();
	return 0;
}