#include <iostream>
#include <mpi.h>
#include <blitz/array.h>

using namespace std;
using namespace blitz;

int main(int argc, char *argv[])
{
	MPI_Init(&argc, &argv);

	int my_id;
	int numprocs;

	int N;
	int local_N;


	Array<float, 1> velocity_input;
	Array<float, 1> velocity;
	Array<float, 1> local_energy;
	Array<float, 1> E;

	MPI_Comm_rank(MPI_COMM_WORLD, &my_id);
	MPI_Comm_size(MPI_COMM_WORLD, &numprocs);

	//Master process reads the data and send it to all other process
	if (my_id == 0) {

		//Continue to take N untill it is divided by numprocs
		while (true) {
			cin >> N;

			if (N%numprocs == 0)
				break;
			else {
				cout << "N (=" << N << ") must be divisible by numprocs (=" << numprocs << ")" << endl;
			}
		}


		velocity_input.resize(N);

		//Input N numbers
		for (int i=0; i<N; i++) {
			cin >> velocity_input(i);
		}

		/*Output array for testing

		cout << N << endl;
		for (int i=0; i<N; i++) {
			cout << velocity_input(i) << "\t";
		}
		cout << endl;
		*/
	}


	//Broadcast N to all procs
	MPI_Bcast(&N, 1, MPI_INT, 0, MPI_COMM_WORLD);

	local_N = N/numprocs;
	velocity.resize(local_N);

	MPI_Scatter(velocity_input.data(), local_N, MPI_INT, velocity.data(), local_N, MPI_INT, 0, MPI_COMM_WORLD);

	/*Output velocity for testing

	for (int p=0; p<numprocs; p++) {
		if (my_id == p) {
			cout << local_N << endl;
			for (int i=0; i<velocity.size(); i++) {
				cout << velocity(i) << "\t";
			}
			cout << endl;
		}
		MPI_Barrier(MPI_COMM_WORLD);
	}
	*/

	//Find local_energy
	local_energy.resize(local_N);

	for (int i=0; i<local_N; i++)
		local_energy(i) = velocity(i)*velocity(i)/2;


	//Collect sum of local_energy from all process in master process
	E.resize(local_N);
	MPI_Reduce(local_energy.data(), E.data(), local_N, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);

	if (my_id == 0)
		cout << "E = " << sum(E) << endl;

	MPI_Finalize();
	return 0;
}